###################################################
### chunk number 1: preliminaries
###################################################
#line 88 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
library("FME")
options(prompt = "> ")
options(width=70)


###################################################
### chunk number 2: 
###################################################
#line 112 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
par(mfrow=c(2,2))
require(FME)


###################################################
### chunk number 3: 
###################################################
#line 118 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
pars <- c(upO2=360,  # concentration at upper boundary, mmolO2/m3
          cons=80,   # consumption rate, mmolO2/m3/day
          ks=1,      # O2 half-saturation ct, mmolO2/m3
          D=1)       # diffusion coefficient, cm2/d


###################################################
### chunk number 4: 
###################################################
#line 125 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
n  <- 100                       # nr grid points
dx <- 0.05   #cm
dX <- c(dx/2,rep(dx,n-1),dx/2)  # dispersion distances; half dx near boundaries
X  <- seq(dx/2,len=n,by=dx)     # distance from upper interface at middle of box


###################################################
### chunk number 5: 
###################################################
#line 135 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
O2fun <- function(pars)
{
  derivs<-function(t,O2,pars)
  {
  with (as.list(pars),{

    Flux <- -D* diff(c(upO2,O2,O2[n]))/dX
    dO2  <- -diff(Flux)/dx-cons*O2/(O2+ks)

    return(list(dO2,UpFlux = Flux[1],LowFlux = Flux[n+1]))
  })
 }

 # Solve the steady-state conditions of the model
 ox <- steady.band(y=runif(n),func=derivs,parms=pars,nspec=1,positive=TRUE)
 data.frame(X=X,O2=ox$y)
}


###################################################
### chunk number 6: 
###################################################
#line 155 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
ox<-O2fun(pars)


###################################################
### chunk number 7: 
###################################################
#line 159 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"



###################################################
### chunk number 8: O2plot
###################################################
#line 161 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
plot(ox$O2,ox$X,ylim=rev(range(X)),xlab="mmol/m3",
     main="Oxygen", ylab="depth, cm",type="l",lwd=2)


###################################################
### chunk number 9: O2plot
###################################################
#line 168 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 161 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#168#"
plot(ox$O2,ox$X,ylim=rev(range(X)),xlab="mmol/m3",
     main="Oxygen", ylab="depth, cm",type="l",lwd=2)
#line 169 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


###################################################
### chunk number 10: 
###################################################
#line 181 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
print(system.time(
Sens2 <- sensRange(parms=pars,func=O2fun,dist="norm",
           num=100,parMean=c(cons=80),parCovar=100)
))


###################################################
### chunk number 11: sens
###################################################
#line 188 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
par(mfrow=c(1,2))
plot(Sens2,xyswap=TRUE,xlab= "O2",
     ylab="depth, cm",main="Sensitivity runs")
plot(summary(Sens2),xyswap=TRUE,xlab= "O2",
     ylab="depth, cm",main="Sensitivity ranges")
par(mfrow=c(1,1))


###################################################
### chunk number 12: sens
###################################################
#line 199 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 188 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#199#"
par(mfrow=c(1,2))
plot(Sens2,xyswap=TRUE,xlab= "O2",
     ylab="depth, cm",main="Sensitivity runs")
plot(summary(Sens2),xyswap=TRUE,xlab= "O2",
     ylab="depth, cm",main="Sensitivity ranges")
par(mfrow=c(1,1))
#line 200 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


###################################################
### chunk number 13: 
###################################################
#line 210 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
O2sens <- sensFun(func=O2fun,parms=pars)


###################################################
### chunk number 14: 
###################################################
#line 216 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
summary(O2sens)


###################################################
### chunk number 15: pairs
###################################################
#line 221 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
pairs(O2sens)


###################################################
### chunk number 16: pairs
###################################################
#line 227 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 221 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#227#"
pairs(O2sens)
#line 228 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


###################################################
### chunk number 17: 
###################################################
#line 235 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
cor(O2sens[,-(1:2)])


###################################################
### chunk number 18: 
###################################################
#line 241 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
Coll <- collin(O2sens)
Coll


###################################################
### chunk number 19: coll
###################################################
#line 245 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
plot(Coll,log="y")


###################################################
### chunk number 20: coll
###################################################
#line 251 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 245 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#251#"
plot(Coll,log="y")
#line 252 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


###################################################
### chunk number 21: 
###################################################
#line 264 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
O2dat <- data.frame(x=seq(0.1,3.5,by=0.1),
    y = c(279,260,256,220,200,203,189,179,165,140,138,127,116,
          109,92,87,78,72,62,55,49,43,35,32,27,20,15,15,10,8,5,3,2,1,0))
O2depth <- cbind(name="O2",O2dat)        # oxygen versus depth
O2flux  <- c(UpFlux=170)                 # measured flux


###################################################
### chunk number 22: 
###################################################
#line 273 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
O2fun2 <- function(pars)
{
  derivs<-function(t,O2,pars)
  {
  with (as.list(pars),{

    Flux <- -D*diff(c(upO2,O2,O2[n]))/dX
    dO2  <- -diff(Flux)/dx-cons*O2/(O2+ks)

    return(list(dO2,UpFlux = Flux[1],LowFlux = Flux[n+1]))
    })
  }

 ox <- steady.band(y=runif(n),func=derivs,parms=pars,nspec=1,
                   positive=TRUE,rtol=1e-8,atol=1e-10)

 list(data.frame(x=X,O2=ox$y),
      UpFlux=ox$UpFlux)
}


###################################################
### chunk number 23: 
###################################################
#line 298 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
Objective <- function (P)
{
 Pars <- pars
 Pars[names(P)]<-P
 modO2 <- O2fun2(Pars)

 # Model cost: first the oxygen profile
 Cost  <- modCost(obs=O2depth,model=modO2[[1]],x="x",y="y")

 # then the flux
 modFl <- c(UpFlux=modO2$UpFlux)
 Cost  <- modCost(obs=O2flux,model=modFl,x=NULL,cost=Cost)

 return(Cost)
}


###################################################
### chunk number 24: 
###################################################
#line 317 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
print(system.time(
sF<-sensFun(Objective,parms=pars)
))
summary(sF)
collin(sF)


###################################################
### chunk number 25: 
###################################################
#line 329 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
collin(sF,parset=c("upO2","cons","ks"))
print(system.time(
Fit<-modFit(p=c(upO2=360,cons=80,ks=1),
                  f=Objective,lower=c(0,0,0))
                  ))
(SFit<-summary(Fit))


###################################################
### chunk number 26: res
###################################################
#line 338 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
plot(Objective(Fit$par),xlab="depth",ylab="",main="residual",legpos="top")


###################################################
### chunk number 27: res
###################################################
#line 344 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 338 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#344#"
plot(Objective(Fit$par),xlab="depth",ylab="",main="residual",legpos="top")
#line 345 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


###################################################
### chunk number 28: 
###################################################
#line 353 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
Pars <- pars
Pars[names(Fit$par)]<- Fit$par
modO2 <- O2fun(Pars)


###################################################
### chunk number 29: fit
###################################################
#line 358 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
plot(O2depth$y,O2depth$x,ylim=rev(range(O2depth$x)),pch=18,
     main="Oxygen-fitted", xlab="mmol/m3",ylab="depth, cm")
lines(modO2$O2,modO2$X)


###################################################
### chunk number 30: fit
###################################################
#line 366 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 358 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#366#"
plot(O2depth$y,O2depth$x,ylim=rev(range(O2depth$x)),pch=18,
     main="Oxygen-fitted", xlab="mmol/m3",ylab="depth, cm")
lines(modO2$O2,modO2$X)
#line 367 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


###################################################
### chunk number 31: 
###################################################
#line 378 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
Covar   <- SFit$cov.scaled * 2.4^2/3
s2prior <- SFit$modVariance


###################################################
### chunk number 32: 
###################################################
#line 383 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
print(system.time(
MCMC <- modMCMC(f=Objective,p=Fit$par,jump=Covar,niter=1000,ntrydr=2,
                var0=s2prior,wvar0=1,updatecov=100,lower=c(NA,NA,0))
))
MCMC$count


###################################################
### chunk number 33: mcmcplot
###################################################
#line 392 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
plot(MCMC,Full=TRUE)


###################################################
### chunk number 34: mcmcplot
###################################################
#line 398 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 392 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#398#"
plot(MCMC,Full=TRUE)
#line 399 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


###################################################
### chunk number 35: mcmchist
###################################################
#line 406 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
hist(MCMC,Full=TRUE)


###################################################
### chunk number 36: mcmchist
###################################################
#line 412 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 406 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#412#"
hist(MCMC,Full=TRUE)
#line 413 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


###################################################
### chunk number 37: mcmcpairs
###################################################
#line 420 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
pairs(MCMC,Full=TRUE)


###################################################
### chunk number 38: mcmcpairs
###################################################
#line 426 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 420 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#426#"
pairs(MCMC,Full=TRUE)
#line 427 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


###################################################
### chunk number 39: 
###################################################
#line 434 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
summary(MCMC)
cor(MCMC$pars)


###################################################
### chunk number 40: mcmcran2
###################################################
#line 440 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
plot(summary(sensRange(parms=pars,parInput=MCMC$par,f=O2fun,num=500)),
  xyswap=TRUE)
points(O2depth$y,O2depth$x)


###################################################
### chunk number 41: mcmcran2
###################################################
#line 448 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"
#line 440 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw#from line#448#"
plot(summary(sensRange(parms=pars,parInput=MCMC$par,f=O2fun,num=500)),
  xyswap=TRUE)
points(O2depth$y,O2depth$x)
#line 449 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEsteady.Rnw"


