###################################################
### chunk number 1: preliminaries
###################################################
#line 90 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
library("FME")
options("prompt" = "R> ", "continue" = "+  ")
options(width=80)
set.seed(1257)


###################################################
### chunk number 2: 
###################################################
#line 237 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
require(diagram)
par(mar=c(0,0,0,0))
cex1 <- 1.5
openplotmat()
names <- c("T-cells (T)","","Infected (I)","Virus (V)")

pos <- matrix(nc=2,byrow=TRUE,
  data=c(0.25,0.75,0.45,0.5, 0.85,0.5,0.25,0.25))
M<-matrix(nr=4, byrow=TRUE, data= c(0,0,0,0,
                                    1,0,0,1,
                                    0,1,0,0,
                                    0,0,1,0))
arr.length<-M
M[[2,1]]<- M[[2,4]] <-""
M[[3,2]]<-"beta~T~V"
M[[4,3]]<-"delta~I"

pp <- straightarrow (pos[1,]+c(0,0.2), pos[1,],arr.pos=0.4,arr.type="triangle")
text(pp[1]+0.035,pp[2],expression(lambda),cex=cex1)
pp<-bentarrow(pos[1,],pos[1,]-c(0.15,0.15), arr.pos=1,arr.type="triangle")
text(pp[1]+0.05,pp[2],expression(rho~ T),cex=cex1)
mv <- pos[4,]
pp<-bentarrow(mv,mv-c(0.12,0.15), arr.pos=1,arr.type="triangle")
text(pp[1]+0.05,pp[2],"c V",cex=cex1)

arr.length[]<-0.4
arr.length[[2,1]]<- arr.length[[2,4]] <-0

pp<-plotmat(M,pos=pos,curve=0,name=names,lwd=1,cex=cex1,
            box.lwd=2,box.size=c(0.1,0,0.1,0.085),arr.type="triangle",
            box.type=c("square","none","square","circle"),  dr=0.001,
            box.prop=c(0.5,0.5,0.5,1),arr.lwd=2,arr.length=arr.length,
            arr.pos=0.4,add=TRUE, box.cex=1.45)
text(pos[4,1]+0.12,pos[4,2]+0.02,"*n",cex=cex1)
par(mar=c(5.1,4.1,4.1,2.1))


###################################################
### chunk number 3: 
###################################################
#line 350 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
HIV_R <- function (pars, V_0 = 50000, dV_0 = -200750, T_0 = 100) {

  derivs <- function(time, y, pars) {
    with (as.list(c(pars, y)), {
      dT <- lam - rho * T - bet * T * V
      dI <- bet * T * V - delt * I
      dV <- n * delt * I - c * V - bet * T * V

      return(list(c(dT, dI, dV), logV = log(V)))
    })
  }

  # initial conditions
  I_0   <- with(as.list(pars), (dV_0 + c * V_0) / (n * delt))
  y     <- c(T = T_0, I = I_0, V = V_0)

  times <- c(seq(0, 0.8, 0.1), seq(2, 60, 2))
  out   <- ode(y = y, parms = pars, times = times, func = derivs)

  as.data.frame(out)
}


###################################################
### chunk number 4: 
###################################################
#line 385 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
HIV <- function (pars, V_0 = 50000, dV_0 = -200750, T_0 = 100) {

  I_0 <- with(as.list(pars), (dV_0 + c * V_0) / (n * delt))
  y <- c(T = T_0, I = I_0, V = V_0)

  times <- c(0, 0.1, 0.2, 0.4, 0.6, 0.8, seq(2, 60, by = 2))
  out <- ode(y = y, parms = pars, times = times, func = "derivshiv",
    initfunc = "inithiv", nout = 1, outnames = "logV", dllname = "FME")

  as.data.frame(out)
}


###################################################
### chunk number 5: 
###################################################
#line 405 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
pars <- c(bet = 0.00002, rho = 0.15, delt = 0.55, c = 5.5, lam = 80, n = 900)
out <- HIV(pars = pars)


###################################################
### chunk number 6: ode
###################################################
#line 410 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
par(mfrow = c(1, 2))
plot(out$time, out$logV, main = "Viral load", ylab = "log(V)",
  xlab = "time", type = "b")
plot(out$time, out$T, main = "CD4+ T", ylab = "-", xlab = "time", type = "b")
par(mfrow = c(1, 1))


###################################################
### chunk number 7: odefig
###################################################
#line 421 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
#line 410 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw#from line#421#"
par(mfrow = c(1, 2))
plot(out$time, out$logV, main = "Viral load", ylab = "log(V)",
  xlab = "time", type = "b")
plot(out$time, out$T, main = "CD4+ T", ylab = "-", xlab = "time", type = "b")
par(mfrow = c(1, 1))
#line 422 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"


###################################################
### chunk number 8: 
###################################################
#line 444 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
DataLogV <- cbind(time = out$time,
            logV = out$logV + rnorm(sd = 0.45, n = length(out$logV)),
            sd = 0.45)


###################################################
### chunk number 9: 
###################################################
#line 453 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
ii    <- which (out$time %in% seq(0, 56, by = 4))
DataT <- cbind(time = out$time[ii],
               T = out$T[ii] + rnorm(sd = 4.5, n = length(ii)),
               sd = 4.5)
head(DataT)


###################################################
### chunk number 10: 
###################################################
#line 512 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
HIVcost <- function (pars) {
  out <- HIV(pars)
  cost <- modCost(model = out, obs = DataLogV, err = "sd")
  return(modCost(model = out, obs = DataT, err = "sd", cost = cost))
}


###################################################
### chunk number 11: 
###################################################
#line 523 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
HIVcost(pars)$model


###################################################
### chunk number 12: cost
###################################################
#line 527 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
plot(HIVcost(pars), xlab="time")


###################################################
### chunk number 13: costfig
###################################################
#line 534 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
plot(HIVcost(pars), xlab="time", legpos="topright")


###################################################
### chunk number 14: sf
###################################################
#line 557 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
# Local sensitivity analysis of bet on logV - code is hidden in text
ref  <- HIV(pars)
pert <- HIV (pars*c(1.1,1,1,1,1,1))
ss   <- (pert$logV-ref$logV)/pert$logV
plot(ref$time,ref$logV,type="l",lwd=2,xlab="time",ylab="logV",
  main="local sensitivity, parameter bet")
lines(pert$time,pert$logV,lwd=2,lty=2)
arrseq <- seq(9,36,6)#c(10,30,50,70,90)
arrows(ref$time[arrseq],ref$logV[arrseq],
  ref$time[arrseq],pert$logV[arrseq], length= 0.075)

legend("bottomright",c("bet=2e-5","bet=2.2e-5"),lty=c(1,2))
par(new=TRUE)
par(fig=c(0.5,0.99,0.5,0.95))
plot(ref$time,ss,type="l",lwd=2,
   xlab="",ylab="",axes=FALSE,frame.plot=TRUE)
points(ref$time[arrseq],ss[arrseq])
title("Sensitivity functions ",line=0.5,cex.main=1)
par(fig=c(0,1,0,1))


###################################################
### chunk number 15: sf
###################################################
#line 582 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
#line 557 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw#from line#582#"
# Local sensitivity analysis of bet on logV - code is hidden in text
ref  <- HIV(pars)
pert <- HIV (pars*c(1.1,1,1,1,1,1))
ss   <- (pert$logV-ref$logV)/pert$logV
plot(ref$time,ref$logV,type="l",lwd=2,xlab="time",ylab="logV",
  main="local sensitivity, parameter bet")
lines(pert$time,pert$logV,lwd=2,lty=2)
arrseq <- seq(9,36,6)#c(10,30,50,70,90)
arrows(ref$time[arrseq],ref$logV[arrseq],
  ref$time[arrseq],pert$logV[arrseq], length= 0.075)

legend("bottomright",c("bet=2e-5","bet=2.2e-5"),lty=c(1,2))
par(new=TRUE)
par(fig=c(0.5,0.99,0.5,0.95))
plot(ref$time,ss,type="l",lwd=2,
   xlab="",ylab="",axes=FALSE,frame.plot=TRUE)
points(ref$time[arrseq],ss[arrseq])
title("Sensitivity functions ",line=0.5,cex.main=1)
par(fig=c(0,1,0,1))
#line 583 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"


###################################################
### chunk number 16: 
###################################################
#line 617 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
Sfun <- sensFun(HIVcost, pars)
summary(Sfun)


###################################################
### chunk number 17: sfun
###################################################
#line 636 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
plot(Sfun, which = c("logV", "T"), xlab="time", lwd = 2)


###################################################
### chunk number 18: sfunfig
###################################################
#line 642 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
#line 636 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw#from line#642#"
plot(Sfun, which = c("logV", "T"), xlab="time", lwd = 2)
#line 643 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"


###################################################
### chunk number 19: sfunp
###################################################
#line 677 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
pairs(Sfun, which = c("logV", "T"), col = c("blue", "green"))


###################################################
### chunk number 20: sfunpfig
###################################################
#line 684 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
#line 677 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw#from line#684#"
pairs(Sfun, which = c("logV", "T"), col = c("blue", "green"))
#line 685 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"


###################################################
### chunk number 21: 
###################################################
#line 750 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
ident <- collin(Sfun)
head(ident, n = 20)


###################################################
### chunk number 22: coll
###################################################
#line 764 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
plot(ident, log = "y")


###################################################
### chunk number 23: collfig
###################################################
#line 771 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
#line 764 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw#from line#771#"
plot(ident, log = "y")
#line 772 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"


###################################################
### chunk number 24: 
###################################################
#line 784 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
collin(Sfun, parset = c("bet", "rho", "delt", "c", "lam", "n"))


###################################################
### chunk number 25: 
###################################################
#line 792 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
collin(Sfun, N = 5)


###################################################
### chunk number 26: 
###################################################
#line 801 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
collin(Sfun, parset = c("bet", "rho", "delt", "c", "lam"), which = "logV")
collin(Sfun, parset = c("bet", "rho", "delt", "c", "lam"), which = "T")


###################################################
### chunk number 27: 
###################################################
#line 835 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
HIVcost2 <- function(lpars)
  HIVcost(c(exp(lpars), n = 900))


###################################################
### chunk number 28: 
###################################################
#line 845 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
Pars <- pars[1:5] * 2
Fit <- modFit(f = HIVcost2, p = log(Pars))
exp(coef(Fit))
deviance(Fit)


###################################################
### chunk number 29: 
###################################################
#line 855 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
ini   <- HIV(pars = c(Pars, n = 900))
final <- HIV(pars = c(exp(coef(Fit)), n = 900))


###################################################
### chunk number 30: fit
###################################################
#line 860 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
par(mfrow = c(1,2))
plot(DataLogV, xlab = "time", ylab = "logV", ylim = c(7, 11))
lines(ini$time, ini$logV, lty = 2)
lines(final$time, final$logV)
legend("topright", c("data", "initial", "fitted"),
   lty = c(NA,2,1), pch = c(1, NA, NA))
plot(DataT, xlab = "time", ylab = "T")
lines(ini$time, ini$T, lty = 2)
lines(final$time, final$T)
par(mfrow = c(1, 1))


###################################################
### chunk number 31: fitfig
###################################################
#line 876 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
#line 860 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw#from line#876#"
par(mfrow = c(1,2))
plot(DataLogV, xlab = "time", ylab = "logV", ylim = c(7, 11))
lines(ini$time, ini$logV, lty = 2)
lines(final$time, final$logV)
legend("topright", c("data", "initial", "fitted"),
   lty = c(NA,2,1), pch = c(1, NA, NA))
plot(DataT, xlab = "time", ylab = "T")
lines(ini$time, ini$T, lty = 2)
lines(final$time, final$T)
par(mfrow = c(1, 1))
#line 877 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"


###################################################
### chunk number 32: 
###################################################
#line 994 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
var0 <- Fit$var_ms_unweighted
cov0 <- summary(Fit)$cov.scaled * 2.4^2/5


###################################################
### chunk number 33:  eval=FALSE
###################################################
## #line 997 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
## MCMC <- modMCMC(f = HIVcost2, p = Fit$par, niter = 5000, jump = cov0,
##                 var0 = var0, wvar0 = 0.1, updatecov = 50)
## #save(MCMC, file="mcmc.Rdata")


###################################################
### chunk number 34: 
###################################################
#line 1004 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
load("mcmc.Rdata")


###################################################
### chunk number 35: 
###################################################
#line 1014 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
options(width = 50)


###################################################
### chunk number 36: 
###################################################
#line 1018 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
MCMC$pars <- exp(MCMC$pars)
summary(MCMC)


###################################################
### chunk number 37: mcmc
###################################################
#line 1034 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
plot(MCMC, Full = TRUE)


###################################################
### chunk number 38: mcmcfig
###################################################
#line 1041 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
par(mar=c(4, 4, 3, 1) + .1)
#line 1034 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw#from line#1042#"
plot(MCMC, Full = TRUE)
#line 1043 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"


###################################################
### chunk number 39: mcmc2
###################################################
#line 1067 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
pairs(MCMC, nsample = 1000)


###################################################
### chunk number 40: mcmcfig2
###################################################
#line 1074 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
pairs(MCMC, nsample = 1000,cex.labels=1.4,cex=0.7)


###################################################
### chunk number 41: 
###################################################
#line 1094 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
sR <- sensRange(func = HIV, parms = pars, parInput = MCMC$par)


###################################################
### chunk number 42: sr
###################################################
#line 1098 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
plot(summary(sR), xlab = "time")


###################################################
### chunk number 43: srfig
###################################################
#line 1105 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
#line 1098 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw#from line#1105#"
plot(summary(sR), xlab = "time")
#line 1106 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"


###################################################
### chunk number 44: 
###################################################
#line 1145 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
parRange <- cbind(min = 0.75 * pars, max = 1.25 * pars)
crlfun <- function (pars)  return(meanVirus = mean(HIV(pars)$V))

CRL <- modCRL(fun = crlfun, parRange = parRange, num = 500)

cor(CRL)[7, ]


###################################################
### chunk number 45: crl
###################################################
#line 1154 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
plot(CRL, ylab = "number of virions", trace = TRUE)


###################################################
### chunk number 46: crlfig
###################################################
#line 1161 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FME.Rnw"
plot(CRL, ylab = "number of virions", cex=0.5, trace = TRUE)


