###################################################
### chunk number 1: preliminaries
###################################################
#line 85 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
library("FME")
options(prompt = "> ")
options(width = 80)


###################################################
### chunk number 2: 
###################################################
#line 139 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
pars <- list(gmax = 0.5, eff = 0.5,
              ks = 0.5, rB = 0.01, dB = 0.01)


###################################################
### chunk number 3: 
###################################################
#line 156 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
solveBact <- function(pars, times=seq(0,50,by=0.5)) {
  derivs <- function(t,state,pars) { # returns rate of change
    with(as.list(c(state,pars)), {

      dBact =  gmax*eff*Sub/(Sub+ks)*Bact - dB*Bact - rB*Bact
      dSub  = -gmax    *Sub/(Sub+ks)*Bact + dB*Bact
      return(list(c(dBact,dSub)))
    })
 }
 state   <- c(Bact = 0.1, Sub = 100)
 ## ode solves the model by integration...
 return(as.data.frame(ode(y=state,times=times,func=derivs,parms=pars)))
}


###################################################
### chunk number 4: 
###################################################
#line 175 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
out <- solveBact(pars)


###################################################
### chunk number 5: ode
###################################################
#line 181 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
plot(out$time,out$Bact,ylim=range(c(out$Bact,out$Sub)),
     xlab="time, hour",ylab="mol C/m3",type="l",lwd=2)
lines(out$time,out$Sub,lty=2,lwd=2)
lines(out$time,out$Sub+out$Bact)

legend("topright",c("Bacteria","Glucose","TOC"),
       lty=c(1,2,1),lwd=c(2,2,1))


###################################################
### chunk number 6: odefig
###################################################
#line 194 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 181 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#194#"
plot(out$time,out$Bact,ylim=range(c(out$Bact,out$Sub)),
     xlab="time, hour",ylab="mol C/m3",type="l",lwd=2)
lines(out$time,out$Sub,lty=2,lwd=2)
lines(out$time,out$Sub+out$Bact)

legend("topright",c("Bacteria","Glucose","TOC"),
       lty=c(1,2,1),lwd=c(2,2,1))
#line 195 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 7: 
###################################################
#line 212 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
parRanges <- data.frame(min=c(0.4,0.4,0.0),max=c(0.6,0.6,0.02))
rownames(parRanges)<- c("gmax","eff","rB")
parRanges


###################################################
### chunk number 8: 
###################################################
#line 225 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
tout    <- 0:50
print(system.time(
sR<-sensRange(func=solveBact,parms=pars,dist="grid",
                sensvar=c("Bact","Sub"),parRange=parRanges[3,],num=50)
))
head(summary(sR))


###################################################
### chunk number 9: sens
###################################################
#line 239 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
par(mfrow=c(2,2))
plot(summary(sR),xlab="time, hour",ylab="molC/m3",
    legpos="topright")
plot(summary(sR),xlab="time, hour",ylab="molC/m3",
     quant=TRUE,col=c("lightblue","darkblue"),legpos="topright")
mtext(outer=TRUE,line=-1.5,side=3,"Sensitivity to rB",cex=1.25)
par(mfrow=c(1,1))


###################################################
### chunk number 10: sensfig
###################################################
#line 252 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 239 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#252#"
par(mfrow=c(2,2))
plot(summary(sR),xlab="time, hour",ylab="molC/m3",
    legpos="topright")
plot(summary(sR),xlab="time, hour",ylab="molC/m3",
     quant=TRUE,col=c("lightblue","darkblue"),legpos="topright")
mtext(outer=TRUE,line=-1.5,side=3,"Sensitivity to rB",cex=1.25)
par(mfrow=c(1,1))
#line 253 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 11: 
###################################################
#line 264 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
Sens2 <- summary(sensRange(func=solveBact,parms=pars,
             dist="latin",sensvar="Bact",parRange=parRanges,num=100))


###################################################
### chunk number 12: sens2
###################################################
#line 269 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
plot(Sens2,main="Sensitivity gmax,eff,rB",xlab="time, hour",ylab="molC/m3")


###################################################
### chunk number 13: sensfig2
###################################################
#line 276 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 269 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#276#"
plot(Sens2,main="Sensitivity gmax,eff,rB",xlab="time, hour",ylab="molC/m3")
#line 277 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 14: 
###################################################
#line 298 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
SnsBact<- sensFun(func=solveBact,parms=pars,
                   sensvar="Bact",varscale=1)
head(SnsBact)


###################################################
### chunk number 15: sfun
###################################################
#line 306 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
plot(SnsBact)


###################################################
### chunk number 16: sfunfig
###################################################
#line 312 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 306 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#312#"
plot(SnsBact)
#line 313 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 17: 
###################################################
#line 326 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
summary(SnsBact)


###################################################
### chunk number 18: 
###################################################
#line 340 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
summary(sensFun(solveBact,pars,varscale=1),var=TRUE)


###################################################
### chunk number 19: 
###################################################
#line 350 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
cor(SnsBact[,-(1:2)])


###################################################
### chunk number 20: pairs
###################################################
#line 354 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
pairs(SnsBact)


###################################################
### chunk number 21: pairsfig
###################################################
#line 361 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 354 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#361#"
pairs(SnsBact)
#line 362 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 22: 
###################################################
#line 379 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
SF <- function (pars) {
  out <- solveBact(pars)
  return(out[nrow(out),2:3])
}
CRL <- modCRL(func=SF,parms=pars,parRange=parRanges[1,])


###################################################
### chunk number 23: crl
###################################################
#line 387 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
plot(CRL)


###################################################
### chunk number 24: crlfig
###################################################
#line 394 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 387 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#394#"
plot(CRL)
#line 395 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 25: 
###################################################
#line 411 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
CRL2 <- modCRL(func=SF,parms=pars,parMean=c("gmax"=0.5,"eff"=0.7),
               parCovar=matrix(nr=2,data=c(0.02,0.02,0.02,0.05)),
               dist="norm",sensvar="Bact",num=150)


###################################################
### chunk number 26: crl2
###################################################
#line 416 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
pairs(CRL2)


###################################################
### chunk number 27: crl2fig
###################################################
#line 422 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 416 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#422#"
pairs(CRL2)
#line 423 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 28: 
###################################################
#line 437 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
Coll <- collin(SnsBact)
Coll
Coll [Coll[,"collinearity"]<20&Coll[,"N"]==4,]
collin(SnsBact,parset=1:5)


###################################################
### chunk number 29: 
###################################################
#line 522 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
Dat<- data.frame(name=c("Obs1","Obs1","Obs2","Obs2"),
                 time=c(1,2,1,2),val=c(50,150,1,2),
                 err=c(5,15,0.1,0.2))
Mod <- Mod <- data.frame(time=0:3,Obs1=rep(4,4),Obs2=1:4)
modCost(mod=Mod,obs=Dat,y="val")


###################################################
### chunk number 30: 
###################################################
#line 532 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
modCost(mod=Mod,obs=Dat,y="val",err="err")


###################################################
### chunk number 31: 
###################################################
#line 542 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
Data <- matrix (nc=2,byrow=2,data=
c(  2,  0.14,    4,  0.21,    6,  0.31,    8,  0.40,
   10,  0.69,   12,  0.97,   14,  1.42,   16,  2.0,
   18,  3.0,    20,  4.5,    22,  6.5,    24,  9.5,
   26, 13.5,    28, 20.5,    30,  29 , 35, 65, 40, 61)
)
colnames(Data) <- c("time","Bact")
head(Data)


###################################################
### chunk number 32: 
###################################################
#line 561 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
Objective <- function(x,parset=names(x)) {
  pars[parset]<- x
  tout    <- seq(0,50,by=0.5)
  ## output times
  out <- solveBact(pars,tout)
  ## Model cost
  return(modCost(obs=Data,model=out))
}


###################################################
### chunk number 33: 
###################################################
#line 577 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
Coll <- collin(sF<-sensFun(func=Objective,parms=pars,varscale=1))
Coll


###################################################
### chunk number 34: coll
###################################################
#line 589 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
plot(Coll,log="y")
abline(h=20,col="red")


###################################################
### chunk number 35: collfig
###################################################
#line 597 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 589 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#597#"
plot(Coll,log="y")
abline(h=20,col="red")
#line 598 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 36: 
###################################################
#line 608 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
collin(sF,parset=1:2)


###################################################
### chunk number 37: 
###################################################
#line 618 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
print(system.time(Fit <- modFit(p=c(gmax=0.5,eff=0.5),
                  f=Objective,lower=c(0.0, 0.0))))
summary(Fit)


###################################################
### chunk number 38: 
###################################################
#line 627 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
pars[c("gmax","eff")]<- Fit$par
  out   <- solveBact(pars)

  Cost  <- modCost(obs=Data,model=out)
  Cost


###################################################
### chunk number 39: fit
###################################################
#line 635 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
plot(out$time,out$Bact,ylim=range(out$Bact),
     xlab="time, hour",ylab="molC/m3",type="l",lwd=2)
 points(Data,cex=2,pch=18)


###################################################
### chunk number 40: fitfig
###################################################
#line 644 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 635 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#644#"
plot(out$time,out$Bact,ylim=range(out$Bact),
     xlab="time, hour",ylab="molC/m3",type="l",lwd=2)
 points(Data,cex=2,pch=18)
#line 645 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 41: res
###################################################
#line 654 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
plot(Cost, xlab="time",ylab="",main="residuals")


###################################################
### chunk number 42: resfig
###################################################
#line 660 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 654 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#660#"
plot(Cost, xlab="time",ylab="",main="residuals")
#line 661 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 43: 
###################################################
#line 696 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
SF<-summary(Fit)
SF
SF[]
Var0<- SF$modVariance
covIni <-SF$cov.scaled *2.4^2/2
MCMC<-modMCMC(p=coef(Fit),f=Objective,jump=covIni,
              var0=Var0,wvar0=1)


###################################################
### chunk number 44: mcmcplot
###################################################
#line 709 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
 plot(MCMC,Full=TRUE)


###################################################
### chunk number 45: mcmcfig
###################################################
#line 716 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 709 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#716#"
 plot(MCMC,Full=TRUE)
#line 717 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 46: mcmcplot2
###################################################
#line 727 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
pairs(MCMC)


###################################################
### chunk number 47: mcmcfig2
###################################################
#line 734 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 727 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#734#"
pairs(MCMC)
#line 735 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 48: 
###################################################
#line 745 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
MC <- as.mcmc(MCMC$pars)


###################################################
### chunk number 49: cumuplot
###################################################
#line 749 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
cumuplot(MC)


###################################################
### chunk number 50: cumuplot
###################################################
#line 755 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 749 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#755#"
cumuplot(MC)
#line 756 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


###################################################
### chunk number 51: 
###################################################
#line 768 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
cov(MCMC$pars)
covIni


###################################################
### chunk number 52: dist
###################################################
#line 780 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
par(mfrow=c(2,2))
Minmax <- data.frame(min=c(1,2),max=c(2,3))
rownames(Minmax) <-c("par1","par2")
Mean   <- c(par1=1.5,par2=2.5)
Covar  <- matrix(nr=2,data=c(2,2,2,3))
plot(Unif(Minmax,100),main="Unif",xlim=c(1,2),ylim=c(2,3))
plot(Grid(Minmax,100),main="Grid",xlim=c(1,2),ylim=c(2,3))
plot(Latinhyper(Minmax,5),main="Latin hypercube",xlim=c(1,2),ylim=c(2,3))
grid()
plot(Norm(parMean=Mean,parCovar=Covar,num=1000),main="multi normal")


###################################################
### chunk number 53: distfig
###################################################
#line 796 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"
#line 780 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw#from line#796#"
par(mfrow=c(2,2))
Minmax <- data.frame(min=c(1,2),max=c(2,3))
rownames(Minmax) <-c("par1","par2")
Mean   <- c(par1=1.5,par2=2.5)
Covar  <- matrix(nr=2,data=c(2,2,2,3))
plot(Unif(Minmax,100),main="Unif",xlim=c(1,2),ylim=c(2,3))
plot(Grid(Minmax,100),main="Grid",xlim=c(1,2),ylim=c(2,3))
plot(Latinhyper(Minmax,5),main="Latin hypercube",xlim=c(1,2),ylim=c(2,3))
grid()
plot(Norm(parMean=Mean,parCovar=Covar,num=1000),main="multi normal")
#line 797 "d:/Rcompile/CRANpkg/local/2.12/FME/inst/doc/FMEdyna.Rnw"


