###################################################
### chunk number 1: preliminaries
###################################################
#line 122 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
library("deSolve")
options(prompt = "> ")
options(width=70)


###################################################
### chunk number 2: 
###################################################
#line 182 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
parameters <- c(a = -8/3,
                b = -10,
                c =  28)


###################################################
### chunk number 3: 
###################################################
#line 193 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
state <- c(X = 1,
           Y = 1,
           Z = 1)


###################################################
### chunk number 4: 
###################################################
#line 223 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
Lorenz<-function(t, state, parameters) {
  with(as.list(c(state, parameters)),{
    # rate of change
    dX <- a*X + Y*Z
    dY <- b * (Y-Z)
    dZ <- -X*Y + c*Y - Z

    # return the rate of change
    list(c(dX, dY, dZ))
  })   # end with(as.list ...
}


###################################################
### chunk number 5: 
###################################################
#line 244 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
times     <- seq(0, 100, by = 0.01)


###################################################
### chunk number 6: 
###################################################
#line 260 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
library(deSolve)
out <- ode(y = state, times = times, func = Lorenz, parms = parameters)
head(out)


###################################################
### chunk number 7: ode
###################################################
#line 276 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
par(oma = c(0, 0, 3, 0))
plot(out, xlab = "time", ylab = "-")
plot(out[, "X"], out[, "Z"], pch = ".")

mtext(outer = TRUE, side = 3, "Lorenz model", cex = 1.5)


###################################################
### chunk number 8: figode
###################################################
#line 287 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 276 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#287#"
par(oma = c(0, 0, 3, 0))
plot(out, xlab = "time", ylab = "-")
plot(out[, "X"], out[, "Z"], pch = ".")

mtext(outer = TRUE, side = 3, "Lorenz model", cex = 1.5)
#line 288 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 9: 
###################################################
#line 317 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
outb <- radau(state, times, Lorenz, parameters, atol = 1e-4, rtol = 1e-4)
outc <- ode(state, times, Lorenz, parameters, method = "radau",
             atol = 1e-4, rtol = 1e-4)


###################################################
### chunk number 10: 
###################################################
#line 336 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
print(system.time(out1 <- rk4   (state, times, Lorenz, parameters)))
print(system.time(out2 <- lsode (state, times, Lorenz, parameters)))
print(system.time(out  <- lsoda (state, times, Lorenz, parameters)))
print(system.time(out  <- lsodes(state, times, Lorenz, parameters)))
print(system.time(out  <- daspk (state, times, Lorenz, parameters)))
print(system.time(out  <- vode  (state, times, Lorenz, parameters)))


###################################################
### chunk number 11: 
###################################################
#line 360 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
rkMethod()


###################################################
### chunk number 12: 
###################################################
#line 370 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
rkMethod("rk23")


###################################################
### chunk number 13: 
###################################################
#line 384 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
func <- function(t, x, parms) {
  with(as.list(c(parms, x)),{
    dP  <- a * P      - b * C * P
    dC  <- b * P * C  - c * C
    res <- c(dP, dC)
    list(res)
  })
}

rKnew <- rkMethod(ID = "midpoint",
  varstep = FALSE,
  A       = c(0, 1/2),
  b1      = c(0, 1),
  c       = c(0, 1/2),
  stage   = 2,
  Qerr    = 1
)

out <- ode(y = c(P = 2, C = 1), times = 0:100, func,
      parms = c(a = 0.1, b = 0.1, c = 0.1), method = rKnew)
head(out)


###################################################
### chunk number 14: 
###################################################
#line 439 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
diagnostics(out1)
diagnostics(out2)


###################################################
### chunk number 15: 
###################################################
#line 445 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
summary(out1)


###################################################
### chunk number 16: 
###################################################
#line 520 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
Aphid <- function(t, APHIDS, parameters) {
  deltax     <- c (0.5, rep(1, numboxes - 1), 0.5)
  Flux       <- -D * diff(c(0, APHIDS, 0)) / deltax
  dAPHIDS    <- -diff(Flux) / delx + APHIDS * r

  # the return value
  list(dAPHIDS )
} # end


###################################################
### chunk number 17: 
###################################################
#line 533 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
D         <- 0.3    # m2/day  diffusion rate
r         <- 0.01   # /day    net growth rate
delx      <- 1      # m       thickness of boxes
numboxes  <- 60

# distance of boxes on plant, m, 1 m intervals
Distance  <- seq(from = 0.5, by = delx, length.out = numboxes)


###################################################
### chunk number 18: 
###################################################
#line 545 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
# Initial conditions:  # ind/m2
APHIDS        <- rep(0, times = numboxes)
APHIDS[30:31] <- 1
state         <- c(APHIDS = APHIDS)      # initialise state variables


###################################################
### chunk number 19: 
###################################################
#line 556 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
times <-seq(0, 200, by = 1)
print(system.time(
  out <- ode.1D(state, times, Aphid, parms = 0, nspec = 1)
))


###################################################
### chunk number 20: 
###################################################
#line 566 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
head(out[,1:5])


###################################################
### chunk number 21: 
###################################################
#line 571 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
summary(out)


###################################################
### chunk number 22: 
###################################################
#line 645 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
daefun <- function(t, y, dy, parameters) {
  res1 <- dy[1] + y[1] - y[2]
  res2 <- y[2] * y[1] - t

  list(c(res1, res2))
}

library(deSolve)
yini  <- c(1, 0)
dyini <- c(1, 0)
times <- seq(0, 10, 0.1)

## solver
print(system.time(out <- daspk(y = yini, dy = dyini,
                           times = times, res = daefun, parms = 0)))


###################################################
### chunk number 23: dae
###################################################
#line 662 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
matplot(out[,1], out[,2:3], type = "l", lwd = 2,
        main = "dae", xlab = "time", ylab = "y")


###################################################
### chunk number 24: figdae
###################################################
#line 669 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 662 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#669#"
matplot(out[,1], out[,2:3], type = "l", lwd = 2,
        main = "dae", xlab = "time", ylab = "y")
#line 670 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 25: 
###################################################
#line 693 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
pendulum <- function (t, Y, parms) {
  with (as.list(Y),
    list(c(u,
           v,
          -lam * x,
          -lam * y - 9.8,
           x^2 + y^2 -1
     ))
  )
}


###################################################
### chunk number 26: 
###################################################
#line 706 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
yini <- c(x = 1, y = 0, u = 0, v = 1, lam = 1)


###################################################
### chunk number 27: 
###################################################
#line 710 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
M <- diag(nrow = 5)
M[5, 5] <- 0
M


###################################################
### chunk number 28: 
###################################################
#line 717 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
index <- c(2, 2, 1)
times <- seq(from = 0, to = 10, by = 0.01)
out  <- radau (y = yini, func = pendulum, parms = NULL,
               times = times, mass = M, nind = index)


###################################################
### chunk number 29: pendulum
###################################################
#line 723 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
plot(out, type = "l", lwd = 2)
plot(out[, c("x", "y")], type = "l", lwd = 2)


###################################################
### chunk number 30: pendulum
###################################################
#line 730 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 723 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#730#"
plot(out, type = "l", lwd = 2)
plot(out[, c("x", "y")], type = "l", lwd = 2)
#line 731 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 31: 
###################################################
#line 755 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
ZODE2 <- function(Time, State, Pars) {
  with(as.list(State), {
       df <- 1i * f
       dg <- -1i * g * g * f
       return(list(c(df, dg)))
  })
}

yini  <- c(f = 1+0i, g = 1/2.1+0i)
times <- seq(0, 2 * pi, length = 100)

out   <- zvode(func = ZODE2, y = yini, parms = NULL, times = times,
  atol = 1e-10, rtol = 1e-10)


###################################################
### chunk number 32: 
###################################################
#line 780 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
analytical <- cbind(f = exp(1i*times), g = 1/(exp(1i*times)+1.1))
tail(cbind(out[,2], analytical[,1]))


###################################################
### chunk number 33: 
###################################################
#line 795 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
f1 <- function  (t, y, parms) {
  ydot <- vector(len = 5)

  ydot[1] <-  0.1*y[1] -0.2*y[2]
  ydot[2] <- -0.3*y[1] +0.1*y[2] -0.2*y[3]
  ydot[3] <-           -0.3*y[2] +0.1*y[3] -0.2*y[4]
  ydot[4] <-                     -0.3*y[3] +0.1*y[4] -0.2*y[5]
  ydot[5] <-                               -0.3*y[4] +0.1*y[5]

  return(list(ydot))
}


###################################################
### chunk number 34: 
###################################################
#line 811 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
yini  <- 1:5
times <- 1:20


###################################################
### chunk number 35: 
###################################################
#line 820 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
out   <- lsode(yini, times, f1, parms = 0, jactype = "fullint")


###################################################
### chunk number 36: 
###################################################
#line 828 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
fulljac <- function  (t, y, parms) {
  jac <- matrix(nrow = 5, ncol = 5, byrow = TRUE,
                data = c(0.1, -0.2,  0  ,  0  ,  0  ,
                        -0.3,  0.1, -0.2,  0  ,  0  ,
                         0  , -0.3,  0.1, -0.2,  0  ,
                         0  ,  0  , -0.3,  0.1, -0.2,
                         0  ,  0  ,  0  , -0.3,  0.1))
  return(jac)
}


###################################################
### chunk number 37: 
###################################################
#line 842 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
out2 <- lsode(yini, times, f1, parms = 0, jactype = "fullusr",
              jacfunc = fulljac)


###################################################
### chunk number 38: 
###################################################
#line 851 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
out3 <- lsode(yini, times, f1, parms = 0, jactype = "bandint",
              bandup = 1, banddown = 1)


###################################################
### chunk number 39: 
###################################################
#line 858 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
bandjac <- function  (t, y, parms) {
  jac <- matrix(nrow = 3, ncol = 5, byrow = TRUE,
                data = c( 0  , -0.2, -0.2, -0.2, -0.2,
                          0.1,  0.1,  0.1,  0.1,  0.1,
                         -0.3, -0.3, -0.3, -0.3,  0))
  return(jac)
}


###################################################
### chunk number 40: 
###################################################
#line 870 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
out4 <- lsode(yini, times, f1, parms = 0, jactype = "bandusr",
              jacfunc = bandjac, bandup = 1, banddown = 1)


###################################################
### chunk number 41: 
###################################################
#line 878 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
out5  <- lsode(yini, times, f1, parms = 0, mf = 10)


###################################################
### chunk number 42: 
###################################################
#line 893 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
eventmod <- function(t, var, parms) {
  list(dvar = -0.1*var)
}

yini <- c(v1 = 1, v2 = 2)
times <- seq(0, 10, by = 0.1)


###################################################
### chunk number 43: 
###################################################
#line 906 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
eventdat <- data.frame(var = c("v1", "v2", "v2", "v1"), time = c(1, 1, 5, 9),
  value = c(1, 2, 3, 4), method = c("add", "mult", "rep", "add"))

eventdat


###################################################
### chunk number 44: 
###################################################
#line 915 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
out <- ode(func = eventmod, y = yini, times = times, parms = NULL,
  events = list(data = eventdat))


###################################################
### chunk number 45: event1
###################################################
#line 920 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
plot(out, type = "l", lwd = 2)


###################################################
### chunk number 46: figevent1
###################################################
#line 926 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 920 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#926#"
plot(out, type = "l", lwd = 2)
#line 927 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 47: 
###################################################
#line 939 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
ballode<- function(t, y, parms) {
  dy1 <- y[2]
  dy2 <- -9.8
  list(c(dy1, dy2))
}


###################################################
### chunk number 48: 
###################################################
#line 951 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
root <- function(t, y, parms) y[1]


###################################################
### chunk number 49: 
###################################################
#line 957 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
event <- function(t, y, parms) {
 y[1]<- 0
 y[2]<- -0.9 * y[2]
 return(y)
}


###################################################
### chunk number 50: 
###################################################
#line 968 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
yini  <- c(height = 0, v = 20)
times <- seq(from = 0, to = 20, by = 0.01)

out   <- lsode(times = times, y = yini, func = ballode, parms = NULL,
  events = list(func = event, root = TRUE), rootfun = root)


###################################################
### chunk number 51: event2
###################################################
#line 976 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
plot(out, which = "height", type = "l",lwd = 2,
  main = "bouncing ball", ylab = "height")


###################################################
### chunk number 52: figevent2
###################################################
#line 983 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 976 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#983#"
plot(out, which = "height", type = "l",lwd = 2,
  main = "bouncing ball", ylab = "height")
#line 984 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 53: 
###################################################
#line 1005 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
library(deSolve)

#-----------------------------
# the derivative function
#-----------------------------
derivs <- function(t, y, parms) {
  if (t < 0)
    lag <- 19
  else
    lag <- lagvalue(t - 0.74)

  dy <- r * y* (1 - lag/m)
  list(dy, dy = dy)
}

#-----------------------------
# parameters
#-----------------------------

r <- 3.5; m <- 19

#-----------------------------
# initial values and times
#-----------------------------

yinit <- c(y = 19.001)
times <- seq(-0.74, 40, by = 0.01)

#-----------------------------
# solve the model
#-----------------------------

yout <- dede(y = yinit, times = times, func = derivs,
             parms = NULL, atol = 1e-10)


###################################################
### chunk number 54: dde
###################################################
#line 1042 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
plot(yout, which = 1, type = "l", lwd = 2, main = "Lemming model", mfrow = c(1,2))
plot(yout[,2], yout[,3], xlab = "y", ylab = "dy", type = "l", lwd = 2)


###################################################
### chunk number 55: figdde
###################################################
#line 1049 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 1042 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#1049#"
plot(yout, which = 1, type = "l", lwd = 2, main = "Lemming model", mfrow = c(1,2))
plot(yout[,2], yout[,3], xlab = "y", ylab = "dy", type = "l", lwd = 2)
#line 1050 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 56: 
###################################################
#line 1085 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
library(deSolve)

combustion <- function (t, y, parms)
  list(y^2 * (1-y) )


###################################################
### chunk number 57: 
###################################################
#line 1091 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
yini  <- 0.01
times <- 0 : 200


###################################################
### chunk number 58: 
###################################################
#line 1095 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
out  <- ode(times = times, y = yini,   parms = 0, func = combustion)
out2 <- ode(times = times, y = yini*2, parms = 0, func = combustion)
out3 <- ode(times = times, y = yini*3, parms = 0, func = combustion)
out4 <- ode(times = times, y = yini*4, parms = 0, func = combustion)


###################################################
### chunk number 59: plotdeSolve
###################################################
#line 1104 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
plot(out, out2, out3, out4, main = "combustion")
legend("bottomright", lty = 1:4, col = 1:4, legend = 1:4, title = "yini*i")


###################################################
### chunk number 60: plotdeSolve
###################################################
#line 1111 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 1104 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#1111#"
plot(out, out2, out3, out4, main = "combustion")
legend("bottomright", lty = 1:4, col = 1:4, legend = 1:4, title = "yini*i")
#line 1112 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 61: 
###################################################
#line 1129 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
head(ccl4data)


###################################################
### chunk number 62: 
###################################################
#line 1133 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
obs <- subset (ccl4data, animal == "A", c(time, ChamberConc))
names(obs) <- c("time", "CP")
head(obs)


###################################################
### chunk number 63: 
###################################################
#line 1142 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
parms <- c(0.182, 4.0, 4.0, 0.08, 0.04, 0.74, 0.05, 0.15, 0.32, 16.17,
            281.48, 13.3, 16.17, 5.487, 153.8, 0.04321671,
            0.40272550, 951.46, 0.02, 1.0,  3.80000000)
yini <- c(AI = 21, AAM = 0, AT = 0, AF = 0, AL = 0, CLT = 0, AM = 0)

out <- ccl4model(times = seq(0, 6, by = 0.05), y = yini, parms = parms)

par2 <- parms
par2[1] <- 0.1
out2 <- ccl4model(times = seq(0, 6, by = 0.05), y = yini, parms = par2)

par3 <- parms
par3[1] <- 0.05
out3 <- ccl4model(times = seq(0, 6, by = 0.05), y = yini, parms = par3)


###################################################
### chunk number 64: plotobs
###################################################
#line 1159 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
plot(out, out2, out3, which = c("AI", "MASS", "CP"),
     col = c("black", "red", "green"), lwd = 2,
     obs = obs, obspar = list(pch = 18, col = "blue", cex = 1.2))
legend("topright", lty = c(1,2,3,NA), pch = c(NA, NA, NA, 18),
  col = c("black", "red", "green", "blue"), lwd = 2,
  legend = c("par1", "par2", "par3", "obs"))


###################################################
### chunk number 65: plotobs
###################################################
#line 1170 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 1159 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#1170#"
plot(out, out2, out3, which = c("AI", "MASS", "CP"),
     col = c("black", "red", "green"), lwd = 2,
     obs = obs, obspar = list(pch = 18, col = "blue", cex = 1.2))
legend("topright", lty = c(1,2,3,NA), pch = c(NA, NA, NA, 18),
  col = c("black", "red", "green", "blue"), lwd = 2,
  legend = c("par1", "par2", "par3", "obs"))
#line 1171 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 66: 
###################################################
#line 1182 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
obs2 <- data.frame(time = 6, MASS = 12)
obs2


###################################################
### chunk number 67: obs2
###################################################
#line 1187 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
plot(out, out2, out3, lwd = 2,
     obs = list(obs, obs2),
     obspar = list(pch = c(16, 18), col = c("blue", "black"),
                   cex = c(1.2 , 2))
     )


###################################################
### chunk number 68: plotobs2
###################################################
#line 1197 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 1187 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#1197#"
plot(out, out2, out3, lwd = 2,
     obs = list(obs, obs2),
     obspar = list(pch = c(16, 18), col = c("blue", "black"),
                   cex = c(1.2 , 2))
     )
#line 1198 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 69: hist
###################################################
#line 1214 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
hist(out, col = grey(seq(0, 1, by = 0.1)), mfrow = c(3, 4))


###################################################
### chunk number 70: plothist
###################################################
#line 1220 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 1214 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#1220#"
hist(out, col = grey(seq(0, 1, by = 0.1)), mfrow = c(3, 4))
#line 1221 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 71: 
###################################################
#line 1243 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
options(prompt = " ")
options(continue = " ")


###################################################
### chunk number 72: 
###################################################
#line 1248 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
lvmod <- function (time, state, parms, N, rr, ri, dr, dri) {
  with (as.list(parms), {
    PREY <- state[1:N]
    PRED <- state[(N+1):(2*N)]

    ## Fluxes due to diffusion
    ## at internal and external boundaries: zero gradient
    FluxPrey <- -Da * diff(c(PREY[1], PREY, PREY[N]))/dri
    FluxPred <- -Da * diff(c(PRED[1], PRED, PRED[N]))/dri

    ## Biology: Lotka-Volterra model
    Ingestion     <- rIng  * PREY * PRED
    GrowthPrey    <- rGrow * PREY * (1-PREY/cap)
    MortPredator  <- rMort * PRED

    ## Rate of change = Flux gradient + Biology
    dPREY    <- -diff(ri * FluxPrey)/rr/dr   +
                GrowthPrey - Ingestion
    dPRED    <- -diff(ri * FluxPred)/rr/dr   +
                Ingestion * assEff - MortPredator

    return (list(c(dPREY, dPRED)))
  })
}


###################################################
### chunk number 73: 
###################################################
#line 1274 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
options(prompt = " ")
options(continue = " ")


###################################################
### chunk number 74: 
###################################################
#line 1279 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
R  <- 20                        # total radius of surface, m
N  <- 100                       # 100 concentric circles
dr <- R/N                       # thickness of each layer
r  <- seq(dr/2,by = dr,len = N) # distance of center to mid-layer
ri <- seq(0,by = dr,len = N+1)  # distance to layer interface
dri <- dr                       # dispersion distances

parms <- c(Da     = 0.05,       # m2/d, dispersion coefficient
           rIng   = 0.2,        # /day, rate of ingestion
           rGrow  = 1.0,        # /day, growth rate of prey
           rMort  = 0.2 ,       # /day, mortality rate of pred
           assEff = 0.5,        # -, assimilation efficiency
           cap    = 10)         # density, carrying capacity



###################################################
### chunk number 75: 
###################################################
#line 1296 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
state    <- rep(0, 2 * N)
state[1] <- state[N + 1] <- 10

times  <- seq(0, 200, by = 1)   # output wanted at these time intervals

print(system.time(
  out <- ode.1D(y = state, times = times, func = lvmod, parms = parms,
                nspec = 2, names = c("PREY", "PRED"),
                N = N, rr = r, ri = ri, dr = dr, dri = dri)
))


###################################################
### chunk number 76: 
###################################################
#line 1309 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
summary(out)


###################################################
### chunk number 77: 
###################################################
#line 1314 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
p10 <- subset(out, select = "PREY", subset = time == 10)
head(p10)


###################################################
### chunk number 78: 
###################################################
#line 1362 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
Simple2D <- function(t, Y, par) {
  y    <- matrix(nrow = nx, ncol = ny, data = Y)  # vector to 2-D matrix
  dY   <- - r_x2y2 * y   # consumption
  return(list(dY))
}


###################################################
### chunk number 79: 
###################################################
#line 1371 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
dy <- dx <- 1  # grid size
nx <- ny <- 100
x  <- seq (dx/2, by = dx, len = nx)
y  <- seq (dy/2, by = dy, len = ny)

# in each grid cell: consumption depending on position
r_x2y2 <- outer(x, y, FUN=function(x,y) ((x-50)^2 + (y-50)^2)*1e-4)


###################################################
### chunk number 80: 
###################################################
#line 1382 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
C <- matrix(nrow = nx, ncol = ny, 1)
ODE3 <- ode.2D(y = C, times = 1:100, func = Simple2D, parms = NULL,
               dimens = c(nx, ny), names = "C", method = "ode45")


###################################################
### chunk number 81: 
###################################################
#line 1388 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
summary(ODE3)
t50 <-  matrix(nrow = nx, ncol = ny,
     data = subset(ODE3, select = "C", subset = (time == 50)))


###################################################
### chunk number 82: twoD
###################################################
#line 1395 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
par(mfrow = c(1, 2))
contour(x, y,  r_x2y2, main = "consumption")
contour(x, y, t50, main = "Y(t = 50)")


###################################################
### chunk number 83: twoD
###################################################
#line 1403 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 1395 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#1403#"
par(mfrow = c(1, 2))
contour(x, y,  r_x2y2, main = "consumption")
contour(x, y, t50, main = "Y(t = 50)")
#line 1404 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


###################################################
### chunk number 84: 
###################################################
#line 1421 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
PCmod <- function(t, x, parms)  {
  with(as.list(c(parms, x)), {
    dP <- c*P   - d*C*P      # producer
    dC <- e*P*C - f*C        # consumer
    res <- c(dP, dC)
    list(res)
  })
}


###################################################
### chunk number 85: 
###################################################
#line 1436 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
parms  <- c(c = 5, d = 0.1, e = 0.1, f = 0.1)


###################################################
### chunk number 86: 
###################################################
#line 1443 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
xstart <- c(P = 0.5, C = 1)
times  <- seq(0, 190, 0.1)

out <- ode(y = xstart, times = times,
  func = PCmod, parms = parms)
tail(out)


###################################################
### chunk number 87: 
###################################################
#line 1469 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
out <- ode(y = xstart,times = times, func = PCmod,
                         parms = parms, atol = 0)
matplot(out[,1], out[,2:3], type = "l", ylab="Producer, Consumer", xlab="time")


###################################################
### chunk number 88: 
###################################################
#line 1528 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
LVmod <- function(Time, State, Pars) {
  with(as.list(c(State, Pars)), {
    Ingestion    <- rIng  * Prey*Predator
    GrowthPrey   <- rGrow * Prey*(1-Prey/K)
    MortPredator <- rMort * Predator

    dPrey        <- GrowthPrey - Ingestion
    dPredator    <- Ingestion*assEff -MortPredator

    return(list(c(dPrey, dPredator)))
  })
}

pars    <- c(rIng   = 0.2,    # /day, rate of ingestion
             rGrow  = 1.0,    # /day, growth rate of prey
             rMort  = 0.2 ,   # /day, mortality rate of predator
             assEff = 0.5,    # -, assimilation efficiency
             K      = 10)     # mmol/m3, carrying capacity

yini    <- c(Prey = 1, Predator = 2)
times   <- seq(0, 200, by = 1)
out     <- ode(func = LVmod, y = yini,
              parms = pars, times = times)



###################################################
### chunk number 89: 
###################################################
#line 1564 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
pars["rIng"] <- 100
out2 <- ode(func = LVmod, y = yini,
           parms = pars, times = times)


###################################################
### chunk number 90: err
###################################################
#line 1603 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
plot(out2, type = "l", lwd = 2, main = "corrupt Lotka-Volterra model")


###################################################
### chunk number 91: err
###################################################
#line 1610 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"
#line 1603 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw#from line#1610#"
plot(out2, type = "l", lwd = 2, main = "corrupt Lotka-Volterra model")
#line 1611 "d:/Rcompile/CRANpkg/local/2.12/deSolve/inst/doc/deSolve.Rnw"


